----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:37:51 04/07/2007 
-- Design Name: 
-- Module Name:    GF16_Mul - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity GF16_Mul is
    Port ( GF16_In_1 : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_In_2 : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_Out : out  STD_LOGIC_VECTOR (3 downto 0));
end GF16_Mul;

architecture Behavioral of GF16_Mul is
-- GF4 Multiplier component.
component GF4_Mul is
    Port ( GF4_In_1 : in  STD_LOGIC_VECTOR (1 downto 0);
           GF4_In_2 : in  STD_LOGIC_VECTOR (1 downto 0);
           GF4_Out : out  STD_LOGIC_VECTOR (1 downto 0));
end component GF4_Mul;

-- Multiplication with constant Theta component.
component GF_Mul_Theta is
    Port ( GF4_In : in  STD_LOGIC_VECTOR (1 downto 0);
           GF4_Out : out  STD_LOGIC_VECTOR (1 downto 0));
end component GF_Mul_Theta;

-- Internal signals.
signal GF4_Mul_Res1, GF4_Mul_Res2,
		 GF4_Mul_Res3, AH_Xor_AL, BH_Xor_BL, 
		 Mul_Theta_Res, AH, AL, BH, BL: STD_LOGIC_VECTOR (1 downto 0);
	
begin
-- Internal routings.
AH <= GF16_In_1(3 downto 2);
AL <= GF16_In_1(1 downto 0);
BH <= GF16_In_2(3 downto 2);
BL <= GF16_In_2(1 downto 0);
AH_Xor_AL <= AH xor AL;
BH_Xor_BL <= BH xor BL;
GF16_Out(3 downto 2) <= GF4_Mul_Res2 xor GF4_Mul_Res3;
GF16_Out(1 downto 0) <= Mul_Theta_Res xor GF4_Mul_Res3;

-- Port mapping.
M1:
GF4_Mul port map(	GF4_In_1 => AH,
						GF4_In_2 => BH,
						GF4_Out => GF4_Mul_Res1	);

M2:
GF4_Mul port map(	GF4_In_1 => AH_Xor_AL,
						GF4_In_2 => BH_Xor_BL,
						GF4_Out => GF4_Mul_Res2	);
						
M3:
GF4_Mul port map(	GF4_In_1 => AL,
						GF4_In_2 => BL,
						GF4_Out => GF4_Mul_Res3	);

Theta:
GF_Mul_Theta port map(	GF4_In => GF4_Mul_Res1,
								GF4_Out => Mul_Theta_Res	);

end Behavioral;

