----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:38:17 04/11/2007 
-- Design Name: 
-- Module Name:    SBox_Comp_Field - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SBox_Comp_Field is
    Port ( GF256_Ori : in  STD_LOGIC_VECTOR (7 downto 0);
           SBox_Out : out  STD_LOGIC_VECTOR (7 downto 0);
			  Reset, Clk : in STD_LOGIC 							);
end SBox_Comp_Field;

architecture Behavioral of SBox_Comp_Field is
-- Isomorphic mapping component.
component Map_Iso is
    Port ( GF256 : in  STD_LOGIC_VECTOR (7 downto 0);
           GFComp : out  STD_LOGIC_VECTOR (7 downto 0));
end component Map_Iso;

-- GF(2^4) Squarrer component.
component GF16_Sqr is
    Port ( GF16_In : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_Out : out  STD_LOGIC_VECTOR (3 downto 0));
end component GF16_Sqr;

-- Constant multiplication by Lambda component.
component GF_Mul_Lambda is
    Port ( GF16_In : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_Out : out  STD_LOGIC_VECTOR (3 downto 0));
end component GF_Mul_Lambda;

-- GF(2^4) Multiplicative inversion component.
component GF16_Mul_Inv is
    Port ( GF16_In : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_Mul_Inv_Out : out  STD_LOGIC_VECTOR (3 downto 0));
end component GF16_Mul_Inv;

-- GF(2^4) Multiplier component.
component GF16_Mul is
    Port ( GF16_In_1 : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_In_2 : in  STD_LOGIC_VECTOR (3 downto 0);
           GF16_Out : out  STD_LOGIC_VECTOR (3 downto 0));
end component GF16_Mul;
-- Combined inverse isomorphic mapping and Affine Transformation component.
component Inv_Iso_Map_AT is
    Port ( GFComp : in  STD_LOGIC_VECTOR (7 downto 0);
           GF256 : out  STD_LOGIC_VECTOR (7 downto 0));
end component Inv_Iso_Map_AT;

-- Internal signals.
signal Iso_Map_Out, Inv_Iso_Map_In, InvMap_AT_out: STD_LOGIC_VECTOR (7 downto 0);
signal 	Iso_Map_Out_H, Iso_Map_Out_L, Iso_Map_HL_Xor,
			Squarrer_Out, Mul_Lambda_Out, Mul_1_Out, Mul_2_Out,
			Mul_3_Out, Mul_Inv_In, Mul_Inv_Out: STD_LOGIC_VECTOR (3 downto 0);
-- Registers.
signal InvMap_AT_reg: STD_LOGIC_VECTOR (7 downto 0) := x"00";
signal 	Iso_Map_Out_H_reg, Mul_Lambda_Out_reg,
			Mul_1_Out_reg, Iso_Map_HL_Xor_reg: STD_LOGIC_VECTOR (3 downto 0) := x"0";


begin
-- Internal routings.
Iso_Map_Out_H <= Iso_Map_Out(7 downto 4);
Iso_Map_Out_L <= Iso_Map_Out(3 downto 0);
Iso_Map_HL_Xor <= Iso_Map_Out_H xor Iso_Map_Out_L;
Mul_Inv_In <= Mul_Lambda_Out_reg xor Mul_1_Out_reg;
Inv_Iso_Map_In <= Mul_3_Out & Mul_2_Out;
SBox_Out <= InvMap_AT_reg;

-- Port mapping.
Iso_Mapping:
Map_Iso port map(	GF256 => GF256_Ori,
						GFComp => Iso_Map_Out	);

Squarrer:
GF16_Sqr port map( GF16_In => Iso_Map_Out_H,
						 GF16_Out => Squarrer_Out	);
						 
Lambda_Mul:
GF_Mul_Lambda port map(	GF16_In => Squarrer_Out,
								GF16_Out => Mul_Lambda_Out	);

Multiplier_1:
GF16_Mul port map( GF16_In_1 => Iso_Map_HL_Xor,
						 GF16_In_2 => Iso_Map_Out_L,
						 GF16_Out => Mul_1_Out			);
						 
Mult_Inverse:
GF16_Mul_Inv port map( GF16_In => Mul_Inv_In,
							  GF16_Mul_Inv_Out => Mul_Inv_Out	);

Multiplier_2:
GF16_Mul port map( GF16_In_1 => Mul_Inv_Out,
						 GF16_In_2 => Iso_Map_HL_Xor_reg,
						 GF16_Out => Mul_2_Out			);

Multiplier_3:
GF16_Mul port map( GF16_In_1 => Mul_Inv_Out,
						 GF16_In_2 => Iso_Map_Out_H_reg,
						 GF16_Out => Mul_3_Out			);

Inv_Iso_Mapping_AT:
Inv_Iso_Map_AT port map( GFComp => Inv_Iso_Map_In,
								 GF256 => InvMap_AT_out			);
-- Pipelining process.
process (Clk)
begin  
   if (Clk'event and Clk = '1') then
      if Reset = '1' then
         InvMap_AT_reg <= x"00";
			Iso_Map_Out_H_reg <= "0000";
			Mul_Lambda_Out_reg <= "0000";
			Mul_1_Out_reg <= "0000";
			Iso_Map_HL_Xor_reg <= "0000";
      else
         InvMap_AT_reg <= InvMap_AT_out;
			Iso_Map_Out_H_reg <= Iso_Map_Out_H;
			Mul_Lambda_Out_reg <= Mul_Lambda_Out;
			Mul_1_Out_reg <= Mul_1_Out;
			Iso_Map_HL_Xor_reg <= Iso_Map_HL_Xor;
      end if;
   end if;
end process;
				
end Behavioral;

